package de.tudresden.inf.st.sample;

import de.tudresden.inf.st.pnml.engine.execution.TransitionCallbackService;
import de.tudresden.inf.st.pnml.engine.ros.DiNeRosNode;
import de.tudresden.inf.st.pnml.engine.transform.PetriNetInitializer;
import de.tudresden.inf.st.pnml.jastadd.model.BalloonCallbackStorage;
import de.tudresden.inf.st.pnml.jastadd.model.BalloonMarking;
import de.tudresden.inf.st.pnml.jastadd.model.PetriNet;
import de.tudresden.inf.st.pnml.jastadd.model.PnmlParser;
import de.tudresden.inf.st.sample.callback.AbstractHandler;
import de.tudresden.inf.st.sample.callback.CallbackMapper;
import de.tudresden.inf.st.sample.communication.Node2Subscriber1;
import de.tudresden.inf.st.sample.nodes.Node1;
import de.tudresden.inf.st.sample.nodes.Node2;
import org.ros.node.DefaultNodeMainExecutor;
import org.ros.node.NodeConfiguration;
import org.ros.node.NodeMainExecutor;
import org.xml.sax.SAXException;

import javax.xml.parsers.ParserConfigurationException;
import java.io.IOException;
import java.net.URI;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <Generated>
 */
public class Main {

    // actual ros / mqtt host url is inserted based on configuration
    private static final String ROS_HOST = "localhost";
    private static final String ROS_MASTER_URI = "http://localhost:11311";
    private static final String MQTT_HOST = "localhost";

    private static final NodeConfiguration nodeConfiguration = NodeConfiguration.newPublic(ROS_HOST);
    private static final NodeMainExecutor nodeMainExecutor = DefaultNodeMainExecutor.newDefault();

    public static void main(java.lang.String[] args) throws IOException, ParserConfigurationException, SAXException {

        /*
         * <Generated>
         * Petri Net base data initialization
         */

        // actual petri net paths are generated
        List<String> petriNetPaths = new ArrayList<>();
        petriNetPaths.add("src/main/resources/nets/publisher-net.pnml");
        petriNetPaths.add("src/main/resources/nets/subscriber-net.pnml");
        List<PetriNet> petriNets = new ArrayList<>();
        Map<String, BalloonMarking> markings = new HashMap<>();
        Map<String, BalloonCallbackStorage> callbackStorages = new HashMap<>();

        for(String petriNetPath : petriNetPaths){
            petriNets.add(PnmlParser.parsePnml(petriNetPath).get(0));
        }

        TransitionCallbackService transitionCallbackService = TransitionCallbackService.getInstance();

        for(PetriNet petriNet : petriNets){
            markings.put(petriNet.getId(), petriNet.initializeBalloonMarking());
            BalloonCallbackStorage storage = petriNet.initializeCallbackStorage();
            transitionCallbackService.init(petriNet.getId(), storage);
            callbackStorages.put(petriNet.getId(), storage);
            PetriNetInitializer.initInputSignalConnections(petriNet, MQTT_HOST, "mqtt");
            PetriNetInitializer.initOutputSignalConnections(petriNet, MQTT_HOST, "mqtt");
        }

        /*
         * <Generated>
         * Callback and token integration
         */

        Map<String, List<AbstractHandler>> callbackMap1 = CallbackMapper.initCallbacksForNode1(petriNets.get(0));

        for (Map.Entry<String, List<AbstractHandler>> entry : callbackMap1.entrySet()) {
            for(AbstractHandler anc : entry.getValue()){
                transitionCallbackService.registerCallback(petriNets.get(0), entry.getKey(), anc);
            }
        }

        Map<String, List<AbstractHandler>> callbackMap2 = CallbackMapper.initCallbacksForNode2(petriNets.get(1));

        for (Map.Entry<String, List<AbstractHandler>> entry : callbackMap2.entrySet()) {
            for(AbstractHandler anc : entry.getValue()){
                transitionCallbackService.registerCallback(petriNets.get(1), entry.getKey(), anc);
            }
        }

        /*
         * <Generated>
         * Start of the Dineros NodescallbackMap1
         */

        nodeConfiguration.setMasterUri(URI.create(ROS_MASTER_URI));

        // generated
        DiNeRosNode node1 = new Node1("node1", petriNets.get(0),
                markings.get(petriNets.get(0).getId()), callbackStorages.get(petriNets.get(0).getId()));
        DiNeRosNode node2 = new Node2("node2", petriNets.get(1),
                markings.get(petriNets.get(1).getId()), callbackStorages.get(petriNets.get(1).getId()));

        /*
        DiNeRosNode diNeRosNode = new DiNeRosDefaultNode("TestNode", petriNets.get(0),
                markings.get(petriNets.get(0).getId()), callbackStorages.get(petriNets.get(0).getId()));
        diNeRosNode.getDinerosServiceServers().put("locB", new DiNeRosDefaultServer());
         */

        node2.getDinerosSubscribers().put("locB", new Node2Subscriber1());

        new Thread(() -> nodeMainExecutor.execute(node1, nodeConfiguration)) {{
            start();
        }};

        new Thread(() -> nodeMainExecutor.execute(node2, nodeConfiguration)) {{
            start();
        }};
    }
}
